"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Vue = exports.Renderer = void 0;
const fs_1 = require("fs");
const path_1 = require("path");
const Vue = require("vue");
exports.Vue = Vue;
const vue_server_renderer_1 = require("vue-server-renderer");
class Renderer {
    constructor(option) {
        this.option = option;
        if (!this.option.dev) {
            this.createInnerRenderer(this.option.dist);
        }
    }
    async render(context) {
        const innerRenderer = await this.getInnerRenderer();
        const requestPath = context.path.replace(/\/$/, '');
        const isPageRequest = requestPath === this.option.route
            || requestPath === this.option.route.replace(/\/index$/, '');
        return new Promise((resolve, reject) => {
            if (!isPageRequest) {
                if (this.devPacker) {
                    return this.devPacker.proxy(context);
                }
            }
            innerRenderer.renderToString({ app: this.app }, (error, res) => {
                if (error) {
                    return reject(error);
                }
                resolve(res);
            });
        });
    }
    async getInnerRenderer() {
        if (this.innerRenderer) {
            return this.innerRenderer;
        }
        if (this.option.dev) {
            const { DevPacker } = await Promise.resolve().then(() => require(Renderer.packer)).catch((error) => {
                if (error.message.search('Cannot find module') >= 0) {
                    /* eslint-disable-next-line no-param-reassign */
                    error.message = `${error.message}. Try: npm i -D ${Renderer.packer}`;
                }
                throw error;
            });
            this.devPacker = new DevPacker({
                rendererType: Renderer.type,
                route: this.option.route,
                dir: this.option.dir,
                template: this.option.template,
                packerOption: this.option.packerOption,
            });
            this.devPacker.on('ready', ({ dist }) => this.createInnerRenderer(dist));
            this.innerRenderer = await this.devPacker.getBuildingRenderer();
            return this.innerRenderer;
        }
        this.createInnerRenderer(this.option.dist);
        return this.innerRenderer;
    }
    createInnerRenderer(dist) {
        try {
            const { route } = this.option;
            const serverDist = path_1.resolve(dist, './server');
            const clientDist = path_1.resolve(dist, './client');
            const serverBundlePath = path_1.resolve(serverDist, 'rie.vue2.runtime.server.json');
            const manifestPath = path_1.resolve(serverDist, 'rie.vue2.runtime.manifest.json');
            const appManifestPath = path_1.resolve(clientDist, `.${route}`, 'rie.vue2.app.manifest.json');
            const appPath = path_1.resolve(serverDist, `.${route}`, 'app.js');
            const template = fs_1.readFileSync(path_1.resolve(serverDist, `.${route}`, 'template.html'), 'utf-8');
            require.cache[serverBundlePath] = undefined;
            require.cache[manifestPath] = undefined;
            require.cache[appManifestPath] = undefined;
            require.cache[appPath] = undefined;
            /* eslint-disable @typescript-eslint/no-require-imports */
            const serverBundle = require(serverBundlePath);
            const manifest = require(manifestPath);
            const appManifest = require(appManifestPath);
            this.app = require(appPath).default;
            /* eslint-enable @typescript-eslint/no-require-imports */
            const clientManifest = this.mergeManifest(manifest, appManifest);
            this.innerRenderer = vue_server_renderer_1.createBundleRenderer(serverBundle, {
                clientManifest,
                template,
                runInNewContext: false,
            });
        }
        catch (exception) {
            console.error(`[${Renderer.type}] createInnerRenderer Error:`);
            console.error(exception);
        }
    }
    mergeManifest(runtime, app) {
        const route = this.option.route.replace(/^\//, '');
        const isDev = this.option.dev;
        const initialFiles = app.initial.filter((file) => file.startsWith(route) || isDev);
        const asyncFiles = app.async.filter((file) => file.startsWith(route) || isDev);
        let clientManifest = {
            publicPath: app.publicPath,
            all: runtime.all.concat(app.all),
            async: runtime.async.concat(asyncFiles),
            initial: runtime.initial.concat(initialFiles),
            modules: {},
        };
        // 如果 runtime 自定义了 publicPath，需要分别为计算成完整路径后，再合并 manifest
        if (runtime.publicPath) {
            const { publicPath: runtimePublicPath } = runtime;
            const { publicPath } = app;
            clientManifest = {
                ...clientManifest,
                publicPath: '',
                all: runtime.all
                    .map(asset => path_1.resolve(runtimePublicPath, asset))
                    .concat(app.all.map(asset => path_1.resolve(publicPath, asset))),
                async: runtime.async
                    .map(asset => path_1.resolve(runtimePublicPath, asset))
                    .concat(asyncFiles.map(asset => path_1.resolve(publicPath, asset))),
                initial: runtime.initial
                    .map(asset => path_1.resolve(runtimePublicPath, asset))
                    .concat(initialFiles.map(asset => path_1.resolve(publicPath, asset))),
            };
        }
        const offset = runtime.all.length;
        Object.keys(runtime.modules).forEach((hash) => {
            clientManifest.modules[hash] = [...runtime.modules[hash]];
        });
        Object.keys(app.modules).forEach((hash) => {
            const assetIds = clientManifest.modules[hash] || [];
            assetIds.push(...app.modules[hash].map(value => value + offset));
            clientManifest.modules[hash] = assetIds;
        });
        return clientManifest;
    }
}
exports.Renderer = Renderer;
Renderer.type = '@riejs/renderer-vue2';
Renderer.packer = '@riejs/packer-vue2';
